@@ -25,8 +25,8 @@ class Converter::DocBook5Converter < Converter::Base
   MANPAGE_SECTION_TAGS = { 'section' => 'refsection', 'synopsis' => 'refsynopsisdiv' }
   TABLE_PI_NAMES = ['dbhtml', 'dbfo', 'dblatex']
 
-  CopyrightRx = /^(.+?)(?: ((?:\d{4}\-)?\d{4}))?$/
-  ImageMacroRx = /^image::?(.+?)\[(.*?)\]$/
+  CopyrightRx = /^(#{CC_ANY}+?)(?: ((?:\d{4}\-)?\d{4}))?$/
+  ImageMacroRx = /^image::?(\S|\S#{CC_ANY}*?\S)\[(#{CC_ANY}+)?\]$/
 
   def initialize backend, opts = {}
     @backend = backend
@@ -35,20 +35,8 @@ class Converter::DocBook5Converter < Converter::Base
 
   def convert_document node
     result = ['<?xml version="1.0" encoding="UTF-8"?>']
-    if node.attr? 'toc'
-      if node.attr? 'toclevels'
-        result << %(<?asciidoc-toc maxdepth="#{node.attr 'toclevels'}"?>)
-      else
-        result << '<?asciidoc-toc?>'
-      end
-    end
-    if node.attr? 'sectnums'
-      if node.attr? 'sectnumlevels'
-        result << %(<?asciidoc-numbered maxdepth="#{node.attr 'sectnumlevels'}"?>)
-      else
-        result << '<?asciidoc-numbered?>'
-      end
-    end
+    result << ((node.attr? 'toclevels') ? %(<?asciidoc-toc maxdepth="#{node.attr 'toclevels'}"?>) : '<?asciidoc-toc?>') if node.attr? 'toc'
+    result << ((node.attr? 'sectnumlevels') ? %(<?asciidoc-numbered maxdepth="#{node.attr 'sectnumlevels'}"?>) : '<?asciidoc-numbered?>') if node.attr? 'sectnums'
     lang_attribute = (node.attr? 'nolang') ? '' : %( xml:lang="#{node.attr 'lang', 'en'}")
     if (root_tag_name = node.doctype) == 'manpage'
       root_tag_name = 'refentry'
@@ -383,9 +371,7 @@ class Converter::DocBook5Converter < Converter::Base
     has_body = false
     result = []
     pgwide_attribute = (node.option? 'pgwide') ? ' pgwide="1"' : ''
-    if (frame = node.attr 'frame', 'all', 'table-frame') == 'ends'
-      frame = 'topbot'
-    end
+    frame = 'topbot' if (frame = node.attr 'frame', 'all', 'table-frame') == 'ends'
     grid = node.attr 'grid', nil, 'table-grid'
     result << %(<#{tag_name = node.title? ? 'table' : 'informaltable'}#{common_attributes node.id, node.role, node.reftext}#{pgwide_attribute} frame="#{frame}" rowsep="#{['none', 'cols'].include?(grid) ? 0 : 1}" colsep="#{['none', 'rows'].include?(grid) ? 0 : 1}"#{(node.attr? 'orientation', 'landscape', 'table-orientation') ? ' orient="land"' : ''}>)
     if (node.option? 'unbreakable')
@@ -413,12 +399,10 @@ class Converter::DocBook5Converter < Converter::Base
       rows.each do |row|
         result << '<row>'
         row.each do |cell|
-          halign_attribute = (cell.attr? 'halign') ? %( align="#{cell.attr 'halign'}") : ''
-          valign_attribute = (cell.attr? 'valign') ? %( valign="#{cell.attr 'valign'}") : ''
           colspan_attribute = cell.colspan ? %( namest="col_#{colnum = cell.column.attr 'colnumber'}" nameend="col_#{colnum + cell.colspan - 1}") : ''
           rowspan_attribute = cell.rowspan ? %( morerows="#{cell.rowspan - 1}") : ''
           # NOTE <entry> may not have whitespace (e.g., line breaks) as a direct descendant according to DocBook rules
-          entry_start = %(<entry#{halign_attribute}#{valign_attribute}#{colspan_attribute}#{rowspan_attribute}>)
+          entry_start = %(<entry align="#{cell.attr 'halign'}" valign="#{cell.attr 'valign'}"#{colspan_attribute}#{rowspan_attribute}>)
           if tsec == :head
             cell_content = cell.text
           else
@@ -499,7 +483,7 @@ class Converter::DocBook5Converter < Converter::Base
     when :link
       %(<link xl:href="#{node.target}">#{node.text}</link>)
     when :bibref
-      %(<anchor#{common_attributes node.id, nil, "[#{node.reftext || node.id}]"}/>#{text})
+      %(<anchor#{common_attributes node.id, nil, (text = "[#{node.reftext || node.id}]")}/>#{text})
     else
       logger.warn %(unknown anchor type: #{node.type.inspect})
       nil
@@ -634,7 +618,7 @@ class Converter::DocBook5Converter < Converter::Base
       if (reftext.include? '<') && ((reftext = reftext.gsub XmlSanitizeRx, '').include? ' ')
         reftext = (reftext.squeeze ' ').strip
       end
-      reftext = (reftext.gsub '"', '&quot;') if reftext.include? '"'
+      reftext = reftext.gsub '"', '&quot;' if reftext.include? '"'
       %(#{attrs} xreflabel="#{reftext}")
     else
       attrs
@@ -742,7 +726,7 @@ class Converter::DocBook5Converter < Converter::Base
       if (cover_image.include? ':') && ImageMacroRx =~ cover_image
         attrlist = $2
         cover_image = doc.image_uri $1
-        unless attrlist.empty?
+        if attrlist
           attrs = (AttributeList.new attrlist).parse ['alt', 'width', 'height']
           if attrs.key? 'scaledwidth'
             # NOTE scalefit="1" is the default in this case
